@@ -425,9 +425,15 @@ class ChainBuilder(object):
 
         chain = self.write_to_mem(addr_for_str, target)
         use_partial_controllers = False
-        # TODO If this fails try using partial controllers
-        chain2 = self.do_syscall(self._execve_syscall, [addr_for_str, 0, 0],
-                                 use_partial_controllers=use_partial_controllers, needs_return=False)
+        try:
+            chain2 = self.do_syscall(self._execve_syscall, [addr_for_str, 0, 0],
+                                     use_partial_controllers=use_partial_controllers, needs_return=False)
+        except RopException:
+            # Try to use partial controllers
+            use_partial_controllers = True
+            l.warning("Trying to use partial controllers for syscall")
+            chain2 = self.do_syscall(self._execve_syscall, [addr_for_str, 0, 0],
+                                     use_partial_controllers=use_partial_controllers, needs_return=False)
         result = chain + chain2
 
         return result
@@ -674,6 +680,8 @@ class ChainBuilder(object):
                     if num_mem_changes == 0 and num_mem_changes2 == 0:
                         if g.stack_change > g2.stack_change:
                             is_good = False
+                        if g.stack_change == g2.stack_change and g.block_length > g2.block_length:
+                            is_good = False
                     if num_mem_changes2 == 0 and num_mem_changes > 0 and g.stack_change >= g2.stack_change:
                         is_good = False
             if not is_good:
